=begin
#-----------------------------------------------------------------------
Creative Commons (cc) Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
you are free to:
Share — copy and redistribute the material in any medium or format
Adapt — remix, transform, and build upon the material
for any purpose, even commercially.
The licensor cannot revoke these freedoms as long as you follow the license terms.
Under the following terms:
Attribution — You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.
ShareAlike — If you remix, transform, or build upon the material, you must distribute your contributions under the same license as the original.
No additional restrictions — You may not apply legal terms or technological measures that legally restrict others from doing anything the license permits.
Name :  	Vico-Section
Inspiration : TIG "Section cut face"  
Author :	Vincent Courteaux vico@regard.org
Usage : 	Créee une scéne avec la section selectionnée avec les bon calques pour faire des plans, coupes, facades.
        	Acessible a partir d'un sous-menu en selectionnant une section
        	Ajoute un menu "mise à jour des sections" dans le menu Extension (Attention cela peut mettre du temps)
        	
History: 1.0 	First release. 20150622 
		 1.1 	2nd release. add a section cut face in layer KS-. 20150813
		 1.2 	add a menu to recalculate all section cut face in one time 20160102
		 1.3	optimisation and remove shadow bug 20160509
		 1.4 	bug active_layer visibility 20161107
      
#-----------------------------------------------------------------------
=end

require 'sketchup.rb'
require 'extensions.rb'

###



module VICO

	def self.initialisation
		
		model = Sketchup.active_model
		layers = model.layers
		model.active_layer = layers["layer0"]
		
		needlayers = ["C-Murs","C-Dalle","C-Men exterieure","C-Men interieure","C-Cloison","C-Isolation","C-Charpente","C-Parement","C-VRD","C-couverture","I-Surfaces","K-1/100-1/50","K-1/200","K-Detail-3D","K-Plan",
					"K-No-Plan","K-tiret","K-masque"]
					
		needlayers.each do |nl|
			if not layers[nl]
				new_layer = layers.add nl
 				new_layer.visible = false
 			end #if
 		end #do
 		
 		styles = Sketchup.active_model.styles
 		
 		needstyles = ["Facade","Trait 1 Coupe 2","Pochage-Calque"]
 		needstyles.each do |ns|
 			if not styles[ns]
				UI.messagebox "erreur! Charger le style " + ns + " dans le fichier"
 			end #if
 		end #do	
	end #init	


	def self.face(section,scolour,sedges,shide,slayer,cutgroup) # (couleur du pochage,avec contour?, avec remplissage?, calque du pochage, groupe qui contiendra le pochage)
	  
		model = Sketchup.active_model
		### start undo
		bounds = model.bounds
	   	entities = model.active_entities ### v1.8 ###
	   	ss = model.selection
	   	view = model.active_view
	   	sfix = Time::now.to_i.to_s[3..-1]
	   
		### get list of all extg faces NOW at start ###v1.2
		extgfaces = []
			for e in entities
			   	if e.typename == "Face"
	    		  extgfaces.push(e)
				end #if
			end #for e
	
		### get data
	   	plane = section.get_plane
	   	bmin  = bounds.min
	   	bminx = bmin.x
	   	bminy = bmin.y
	   	bminz = bmin.z
	   	bmax  = bounds.max
	   	bmaxx = bmax.x
	   	bmaxy = bmax.y
	   	bmaxz = bmax.z
		### make group
	   	newgroup=entities.add_group
	   	newgroupentities=newgroup.entities
	   	c1 = [bminx, bminy, bminz]
	   	c2 = [bmaxx, bminy, bminz]
	   	c3 = [bminx, bminy, bmaxz]
	   	c4 = [bmaxx, bminy, bmaxz]
	   	c5 = [bminx, bmaxy, bmaxz]
	   	c6 = [bmaxx, bmaxy, bmaxz]
	   	c7 = [bminx, bmaxy, bminz]
	   	c8 = [bmaxx, bmaxy, bminz]
	   	e1 = newgroupentities.add_edges [ c1, c2 ]
	   	e2 = newgroupentities.add_edges [ c3, c4 ]
	   	e3 = newgroupentities.add_edges [ c5, c6 ]
	   	e4 = newgroupentities.add_edges [ c7, c8 ]
	   	e5 = newgroupentities.add_edges [ c1, c3 ]
	   	e6 = newgroupentities.add_edges [ c2, c4 ]
	   	e7 = newgroupentities.add_edges [ c7, c5 ]
	   	e8 = newgroupentities.add_edges [ c8, c6 ]
	   	e9 = newgroupentities.add_edges [ c1, c7 ]
	   	e10 = newgroupentities.add_edges [ c3, c5 ]
	   	e11 = newgroupentities.add_edges [ c4, c6 ]
	   	e12 = newgroupentities.add_edges [ c2, c8 ]
	   	line1 = e1[0].line
	   	line2 = e2[0].line
	   	line3 = e3[0].line
	   	line4 = e4[0].line
	   	line5 = e5[0].line
	   	line6 = e6[0].line
	   	line7 = e7[0].line
	   	line8 = e8[0].line
	   	line9 = e9[0].line
	   	line10 = e10[0].line
	   	line11 = e11[0].line
	   	line12 = e12[0].line
	   	e1[0].erase!
	   	e2[0].erase!
	   	e3[0].erase!
	   	e4[0].erase!
	   	e5[0].erase!
	   	e6[0].erase!
	   	e7[0].erase!
	   	e8[0].erase!
	   	e9[0].erase!
	   	e10[0].erase!
	   	e11[0].erase!
	   	e12[0].erase!
		### find intersects with plane
	   	p1 = Geom.intersect_line_plane line1, plane
	   	p2 = Geom.intersect_line_plane line2, plane
	   	p3 = Geom.intersect_line_plane line3, plane
	   	p4 = Geom.intersect_line_plane line4, plane
	   	p5 = Geom.intersect_line_plane line5, plane
	   	p6 = Geom.intersect_line_plane line6, plane
	   	p7 = Geom.intersect_line_plane line7, plane
	   	p8 = Geom.intersect_line_plane line8, plane
	   	p9 = Geom.intersect_line_plane line9, plane
	   	p10 = Geom.intersect_line_plane line10, plane
	   	p11 = Geom.intersect_line_plane line11, plane
	   	p12 = Geom.intersect_line_plane line12, plane
		if p1 ### NOT z 
	   		e1 = newgroupentities.add_line p1,p2
	   		e2 = newgroupentities.add_line p2,p3
	   		e3 = newgroupentities.add_line p3,p4
	   		e4 = newgroupentities.add_line p4,p1
		elsif  p5 ### in z
	   		e1 = newgroupentities.add_line p5,p6
	   		e2 = newgroupentities.add_line p6,p8
	   		e3 = newgroupentities.add_line p8,p7
	   		e4 = newgroupentities.add_line p7,p5
		else
	   		e1 = newgroupentities.add_line p9,p10
	   		e2 = newgroupentities.add_line p10,p11
	   		e3 = newgroupentities.add_line p11,p12
	   		e4 = newgroupentities.add_line p12,p9
		end #if p1 etc
		###
	   	face = newgroupentities.add_face [e1,e2,e3,e4]
		### we now have a suitable face at the plane...
		###
		
		###
		cutgroupentities = cutgroup.entities
		### ###################################################
		xgroup = entities.add_group
		xgroupentities = xgroup.entities
		xgroup.name= "XSgroup"
		faces=[]
		for entity in ss
	   		faces.push(entity) if entity.typename == "Face"
		end#end entity
		loops=[]
		for face in faces
	   		loops.push(face.loops)
		end#face
		for floop in loops
			for loop in floop
	   			edges=loop.edges
	   			startends=[]
	   			for edge in edges
	      			startends.push([edge.start,edge.end])
	   			end#for edge
	   			lines=[]
	   			for verts in startends
	      			lines.push(xgroupentities.add_line(verts))
	   			end#for verts
	   			###xgroupentities.add_face(lines)
	   			for line in lines
	    	  		line.find_faces
	   			end#for line
	 		end#for loop
		end#for floop
		faces=[]
		for entity in xgroupentities
	   		faces.push(entity) if entity.typename == "Face"
		end#end entity
		### ###########################################
		### now work out which is which to keep...
		keptfaces = []
		(faces.length-1).times do |this|
		   for face in faces
		    if face.valid?
		      for edgeuse in face.outer_loop.edgeuses
		         if not edgeuse.partners[0] ### outermost face
		            keptfaces.push(face)
		            faces = faces - [face]
		            loops = face.loops
		            for loop in loops
		               for fac in faces
		                  if fac.valid? and (fac.outer_loop.edges - loop.edges) == []
		                     faces = faces - [fac]
		                     fac.erase!### fac abutts kept face so it must be erased...
		                  end #if fac
		               end #for fac
		            end #for loop
		         end #if outermost
		      end #for edgeuse
		    end #if valid
		   end #for face
		end#times
		###
		entities.intersect_with true, newgroup.transformation, cutgroup, cutgroup.transformation, false, newgroup
		###
		###xgroup.erase! ### set it back as was... ###v1.1
		newgroup.erase!
		###
		### we now have cross section edges in 'newcutgroup' - now - make faces...
		for e in cutgroupentities
		   if e.typename == "Edge"
		      e.find_faces
		   end
		end #for e
		###
		### now we process the faces and loops...
		faces = []
		for f in cutgroupentities
		   if f.typename == "Face"
		      faces.push(f)
		   end
		end #for f
		faces = faces.uniq
		###
		for face in faces
		   face.reverse! if face.normal == [ plane[0], plane[1], plane[2] ] ### now all face out of cut
		end #for face
		### now work out which is which...
		keptfaces = []
		(faces.length-1).times do |this|
		   for face in faces
		    if face.valid?
		      for edgeuse in face.outer_loop.edgeuses
		         if not edgeuse.partners[0] ### outermost face
		            keptfaces.push(face)###v2.0
		            faces = faces - [face]
		            loops = face.loops
		            for loop in loops
		               for fac in faces
		                  if fac.valid? and (fac.outer_loop.edges - loop.edges) == []
		                     faces = faces - [fac]
		                     fac.erase!### fac abutts kept face so it must be erased...
		                  end #if fac
		               end #for fac
		            end #for loop
		         end #if outermost
		      end #for edgeuse
		    end #if valid
		   end #for face
		end # times
		### erase any stray edges... v1.2 ### doing it x2 works best ???
		(cutgroupentities.length - 1).times do |something|
		  for e in cutgroupentities
		   if e.typename == "Edge" and sedges == "No" ###v2.2
		      e.erase! if not e.faces[0]
		   end #if
		  end #for e
		end #times ###v1.2
		### move cut into view ~0.2mm ### v2.0/v2.1
		vector=Geom::Vector3d.new(plane[0],plane[1],plane[2])###faces[0].normal.normalize!
		tpoint=ORIGIN.offset(vector,1.mm)###v2.1
		transf = Geom::Transformation.new tpoint
		cutgroup.move! transf
		### do colours / layer / hidden
		 for f in cutgroupentities
		   if f.typename == "Face"
		      if scolour == "<Default>" ###v1.3
		         f.material= nil        ###v1.3
		      else                       ###v1.3
		         f.material= scolour
		      end #if                    ###v1.3
		   end #if
		   if f.typename == "Edge" and shide == "Yes" ###v1.4 ###v2.2
		         f.hidden= true         ###v1.4
		   end #if                       ###v1.4
		   f.layer= nil  ###v2.2 rather than Layer0
		 end #for f
		
		###
	  	cutgroup.layer = slayer
	  	
		
	end #def VICO.face


	def self.selection_section
    	model = Sketchup.active_model
    	ss = model.selection
    	ss.each do |ee|
			return nil if not ee.instance_of? Sketchup::SectionPlane
    	end
	end
	
		
	def self.section_calques(section,calques,s_group)
		model = Sketchup.active_model
		entities = model.active_entities
		layers = model.layers
		
		calques.each do |cal|
			### make new group for cut
			papa = entities.add_group
			papaentities = papa.entities
			papa.name = "temp"
			
			layers["K-1/200"].visible = false
			layers["K-1/100-1/50"].visible = false
			
			for layer in layers
  				if layer.name.chars.first == "C"
  				layer.visible = false
  				end #if
  			end #for
  			
  			layers[cal].visible = true
  			color = layers[cal].color
  			
			if cal[0..4] == "C-Men" #menuiserie
				layers["K-1/100-1/50"].visible = true
				layers["K-1/200"].visible = false
				VICO.face(section,color,"Yes","No","K-1/100-1/50",papa)
			else
				VICO.face(section,color,"Yes","No",cal,papa)
			end
			
			s_group.entities.add_instance(papa.entities.parent, papa.transformation*s_group.transformation)
			papa.entities.parent.instances[1].material=papa.material
			papa.entities.parent.instances[1].layer=papa.layer
			papa.erase!
		end #do
	end #section	
			
	
	def self.quel_nom
    	prompts = ["nom :"]
    	defaults = []
    	input = UI.inputbox prompts, defaults, "Entrer le nom de la façade"
		if input
			return input[0].strip 
		else
			return nil
		end	
	end


	def self.nom_coupe
    	prompts = ["nom :"]
    	defaults = []
    	input = UI.inputbox prompts, defaults, "Entrer le nom de la coupe (AA/BB/CC/..."
		if input
			return input[0].strip 
		else
			return nil
		end	
	end


	def self.quel_niveau
    	prompts = ["niveau :"]
    	defaults = []
    	input = UI.inputbox prompts, defaults, "Entrer le niveau (-1/0/1/...)"
		if input
			return "N" + input[0].strip 
		else			return nil		end	
	end


	def self.plan_horizontal?(plane)
	  	snormal = Geom::Vector3d.new(plane[0], plane[1], plane[2])
	  	snormal.parallel? Z_AXIS
	end
	
	
	def self.mise_a_jour
	
		model = Sketchup.active_model
		layers = model.layers
		pages = model.pages
		
		sectionlist = model.entities.find_all{|sel| sel.kind_of? Sketchup::SectionPlane }
		selsecnamelist = sectionlist.map{|sel| sel.get_attribute( "SECTION_SPEC","GROUP","" ) }
		
		if selsecnamelist.size > 0
			selsecnamelist.push "ALL"
			prompts = [ "Nom des Section" ]
			lists = [ selsecnamelist.join("|") ]
			results = UI.inputbox prompts, [ "Tout" ] , lists , "Selectinner les sections"
			if results != nil
				secname = results[0]
				if secname == "Tout"
					selsecs = sectionlist
				else
					selsecs = sectionlist.find_all{|sel| sel.get_attribute( "SECTION_SPEC","GROUP","" ) == secname }
				end
			end
		else
				
			UI.messagebox "Here is No Sections for this tool."
			return
		end
		
		model.start_operation("Mise à jour des sections",true)
		selsecs.to_a.each{|sel| 
				secname = sel.get_attribute( "SECTION_SPEC","GROUP","" )
				entities = model.entities
				
				layer_name = "KS-" + secname
				layers.to_a.each{|l| if l.name==layer_name
									layers.remove(l, true)
							 end }
							 
				hlayers = ["C-Murs","C-Cloison","C-Isolation","C-Dalle","C-Charpente","C-Men exterieure","C-Men interieure","C-Parement","C-VRD","C-couverture"]
				VICO.Section(sel,secname,hlayers,1)
				}
		model.commit_operation
	
	end #mise à jour
	
	
	def self.Section(section,nom,q_layers,mise_jour)
	
		model = Sketchup.active_model
		VICO.initialisation
		entities = model.active_entities
 		layers = model.layers
 		pages = model.pages
 		styles = model.styles
 		page_encours = pages.selected_page
 		layers_encours = layers.find_all{|l| l.visible? }
 		 		 		 		
 		k_layers = layers.find_all{|l| l.name[0..0]=="K" }
 		c_layers = layers.find_all{|l| l.name[0..0]=="C" }
 		i_layers = layers.find_all{|l| l.name[0..0]=="I" }
 		e_layers = layers.find_all{|l| l.name[0..0]=="E" }
		k_layers.to_a.each{|l| l.visible = false }
		i_layers.to_a.each{|l| l.visible = false }
		e_layers.to_a.each{|l| l.visible = false }
 		
     	section_group = entities.add_group
		section_groupentities = section_group.entities
		section_group.name = "Section_" + nom
		layer_name = "KS-" + nom
		layers.to_a.each{|l| if l.name==layer_name
									layers.remove(l, true)
							 end }
		new_layer = layers.add("KS-" + nom)
		new_layer.page_behavior = LAYER_IS_HIDDEN_ON_NEW_PAGES
		pages.each { |p| p.set_visibility( new_layer, false ) }
 		new_layer.visible = true
		section_group.layer = new_layer
		
		VICO.section_calques(section,q_layers,section_group)
							
		if mise_jour == 1
		
			pages = model.pages
			if nom[0..1]=="N-"	
				find_pages = pages.find_all{|page| page.name[0..2]==nom }
				find_pages.to_a.each{|e| e.set_visibility new_layer,true }
			else 
				find_pages = pages.find_all{|page| page.name[0..1]==nom }
				find_pages.to_a.each{|e| e.set_visibility new_layer,true }
			end
								
		else
			pages = model.pages
			
			if nom[0..1]=="N-"	
				find_pages = pages.find_all{|page| page.name[0..2]==nom }
				find_pages.to_a.each{|e| pages.erase(e) }
			else 
				find_pages = pages.find_all{|page| page.name[0..1]==nom }
				find_pages.to_a.each{|e| pages.erase(e) }
			end
			
			
					
			# S1 trait epais
		
			page = pages.add nom + " 1"
			label = page.label
			description = page.description = "Trait épais"
			pages.selected_page = page
				
			# Gére la visibilité des calques
 			layers = model.layers
 		
 			active_layers = layers.find_all{|l| l.visible? }
 		
 			layers.to_a.each{|l| page.set_visibility l,false }
 		
 			page.set_visibility layers["C-Murs"],true
 			page.set_visibility layers["C-Dalle"],true
 			page.set_visibility layers["C-VRD"],true
 			page.set_visibility new_layer,true 		
 		
 			# Gére le style
     		styles = Sketchup.active_model.styles
     		status = styles.add_style "Trait 1 Coupe 2", true
     		styles.selected_style = styles["Trait 1 Coupe 2"]
     			    	     	
			# Crée la vues
		
			view = model.active_view
  			camera = view.camera
			camera.perspective = false
			up = camera.up
			eye = camera.eye
			section.activate
			splane = section.get_plane
		
			eye_on_splane = eye.project_to_plane splane
			target = camera.target
			target.set!(eye_on_splane) 
  			if VICO.plan_horizontal?(splane)
	  			up.set!(1, 0, 0)  # down
			else	
  				up.set!(0, 0, 1)  # level
			end
  			camera.set(eye, target, up)
  			view.camera = camera
			view.zoom_extents
			model.rendering_options['DisplaySectionPlanes'] = false
			Sketchup.active_model.shadow_info['DisplayShadows']=false
			Sketchup.active_model.rendering_options["DisplayFog"] = false
			page.update
			name = page.name
		
								
			# P2 trait fin et moyen
			
 			# Gére le style
     		styles = Sketchup.active_model.styles
     		status = styles.add_style "Trait 1 Coupe 2", true
     		styles.selected_style = styles["Trait 1 Coupe 2"]
	    	     	
			# Crée la vues
			page = pages.add nom + " 2"
			description = page.description = "Plan trait moyen"
				
			# Gére la visibilité des calques
		
	 		page.set_visibility layers["C-Murs"],false
	 		page.set_visibility layers["C-Dalle"],false
 			page.set_visibility layers["C-VRD"],false
 			page.set_visibility layers["C-Cloison"],true
 			page.set_visibility layers["C-Isolation"],true
 			page.set_visibility layers["C-Charpente"],true
 			page.set_visibility layers["C-Parement"],true
 			if nom[0..0] == "N"
 				page.set_visibility layers["I-Surfaces"],true
 			else
 				page.set_visibility layers["I-Surfaces"],false
 			end
 			page.set_visibility new_layer,true 		
 		 		
 			model.rendering_options['DisplaySectionPlanes'] = false
 			Sketchup.active_model.shadow_info['DisplayShadows']=false
 			
			page.update
			
				
			# P3 
		
			page = pages.add nom + " 3"
			description = page.description = "Trait fin 1/200"
				
			# Gére la visibilité des calques
		
			active_layers.to_a.each{|l| page.set_visibility l,true}
			c_layers.to_a.each{|l| page.set_visibility l,true }
			page.set_visibility new_layer,true
			page.set_visibility layers["K-1/200"],true
			page.set_visibility layers["K-1/100-1/50"],false
			page.set_visibility layers["I-Surfaces"],false
			page.set_visibility layers["K-Detail-3D"],false
			
			if nom[0..0] == "N" # Affichage plan ou coupe
				page.set_visibility layers["K-Plan"],true
				page.set_visibility layers["K-No-Plan"],false
			else
				page.set_visibility layers["K-Plan"],false
				page.set_visibility layers["K-No-Plan"],true
			end
 		 		
 			model.rendering_options['DisplaySectionPlanes'] = false
 			Sketchup.active_model.shadow_info['DisplayShadows']=false
 			
			page.update
			
			
			# P5 
		
			page = pages.add nom + " 5"
			description = page.description = "Trait fin"
				
			# Gére la visibilité des calques
		
			active_layers.to_a.each{|l| page.set_visibility l,true}
			c_layers.to_a.each{|l| page.set_visibility l,true }
			page.set_visibility new_layer,true
			page.set_visibility layers["K-1/200"],false
			page.set_visibility layers["K-1/100-1/50"],true
			page.set_visibility layers["I-Surfaces"],false
			page.set_visibility layers["K-Detail-3D"],false
			
			if nom[0..0] == "N" # Affichage plan ou coupe
				page.set_visibility layers["K-Plan"],true
				page.set_visibility layers["K-No-Plan"],false
			else
				page.set_visibility layers["K-Plan"],false
				page.set_visibility layers["K-No-Plan"],true
	 	 	end
	 	 			 		
	 		model.rendering_options['DisplaySectionPlanes'] = false
	 		Sketchup.active_model.shadow_info['DisplayShadows']=false
	 		
 			page.update	
				
			# P4 Pochage
			
	 		# Gére le style
	     	styles = Sketchup.active_model.styles
  		   	status = styles.add_style "Pochage-Calque", true
     		styles.selected_style = styles["Pochage-Calque"]
	    	     	
			# Crée la vues
			page = pages.add nom + " 4"
			description = page.description = "Plan Pochage"
				
			# Gére la visibilité des calques
		
			layers = model.layers
			layers.to_a.each{|l| page.set_visibility l,false }
			q_layers.to_a.each{|hl| page.set_visibility layers[hl],true }
			page.set_visibility layers["K-1/100-1/50"],true
			page.set_visibility new_layer,true
 		 		 		
 			model.rendering_options['DisplaySectionPlanes'] = false
 			Sketchup.active_model.shadow_info['DisplayShadows']=false		
			page.update	
				
			
		end #if
		pages.selected_page = page_encours
		layers_encours.to_a.each{|l| l.visible = true}
			
			
		

	end
	
	
	def self.Coupe # Coupe  
  		model = Sketchup.active_model
  		pages = model.pages
  		layers = model.layers
  		active_layers = layers.find_all{|l| l.visible? }
  		
  		model.start_operation("creation d'une coupe",true) 
		section = model.selection[0]
		model.selection.clear
				
		VICO.initialisation
	  	
	  	secname = section.get_attribute( "SECTION_SPEC","GROUP" )
		layers = model.layers
		hlayers = ["C-Murs","C-Cloison","C-Isolation","C-Dalle","C-Charpente","C-Men exterieure","C-Men interieure","C-Parement","C-VRD","C-couverture"] #calques dont on as besoin en plan de section
				
		if secname == nil
	  		c_nom = nom_coupe
  			return unless c_nom
				if c_nom == ""
					UI.messagebox c_nom + "nom invalide"
				return
				end
			section.set_attribute("SECTION_SPEC","GROUP",c_nom)
			VICO.Section(section,c_nom,hlayers,0)
			
		else 
			c_nom = secname
			result = UI.messagebox( "mise à jour?" , MB_YESNO)
			if result == IDYES
				VICO.Section(section,c_nom,hlayers,1)
				
			else
				c_nom = nom_coupe
  				return unless c_nom
					if c_nom == ""
						UI.messagebox c_nom + "nom invalide"
					return
					end
				section.set_attribute("SECTION_SPEC","GROUP",c_nom)
				VICO.Section(section,c_nom,hlayers,0)
			end
		end
		model.commit_operation
		
		
	end # Coupe

	
	def self.Plan  # plan  
  		model = Sketchup.active_model
  		model.start_operation("Creation d'un Plan",true)
		section = model.selection[0]
		
		model.selection.clear 
		
		VICO.initialisation
			
	  	secname = section.get_attribute( "SECTION_SPEC","GROUP" )
		layers = model.layers
		hlayers = ["C-Murs","C-Cloison","C-Isolation","C-Charpente","C-Men exterieure","C-Men interieure","C-Parement","C-VRD","C-couverture"] #calques dont on as besoin en plan de section
		
		if secname == nil
	  		p_niveau = quel_niveau
  			return unless p_niveau
				if p_niveau == "N"
					UI.messagebox p_niveau + "nom invalide"
				return
				end
			section.set_attribute("SECTION_SPEC","GROUP",p_niveau)
			VICO.Section(section,p_niveau,hlayers,0)
			
		else 
			p_niveau = secname
			result = UI.messagebox( "mise à jour?" , MB_YESNO)
			if result == IDYES
				VICO.Section(section,p_niveau,hlayers,1)
			else
				p_niveau = quel_niveau
  				return unless p_niveau
					if p_niveau == "N"
						UI.messagebox p_niveau + "nom invalide"
					return
					end
				section.set_attribute("SECTION_SPEC","GROUP",p_niveau)
				VICO.Section(section,p_niveau,hlayers,0)
			end
		end
		model.commit_operation
		
	end # plan
	
	
	def self.Facade
  		model = Sketchup.active_model
		model.start_operation("Creation de Facade",true)
		section = model.selection[0]
		model.selection.clear
							
		VICO.initialisation
	  			secname = section.get_attribute( "SECTION_SPEC","GROUP" )
		
		if secname == nil
		
	  		f_nom = quel_nom
  			return unless f_nom
				if f_nom == ""
					UI.messagebox f_nom + "nom invalide"
					return
				end
				section.set_attribute("SECTION_SPEC","GROUP",f_nom)
				m_jour=0
							
		else 
			f_nom = secname
			result = UI.messagebox( "mise à jour?" , MB_YESNO)
			if result == IDYES
				m_jour=1
			else
				f_nom = quel_nom
  				if f_nom == ""
					UI.messagebox f_nom + "nom invalide"
					return
				end
				section.set_attribute("SECTION_SPEC","GROUP",f_nom)
				m_jour=0
				
			end
		end
		
		
		if m_jour==0
				
		pages = model.pages
		page = pages.add f_nom
		description = page.description = "façade"
		pages.selected_page = page		
				
				
		# Gére la visibilité des calques	
		
		layers = model.layers
		
		k_layers = layers.find_all{|l| l.name[0..0]=="K" }
 		c_layers = layers.find_all{|l| l.name[0..0]=="C" }
 		i_layers = layers.find_all{|l| l.name[0..0]=="I" }
 		e_layers = layers.find_all{|l| l.name[0..0]=="E" }
		k_layers.to_a.each{|l| l.visible = false }
		c_layers.to_a.each{|l| l.visible = true }
		i_layers.to_a.each{|l| l.visible = false }
		e_layers.to_a.each{|l| l.visible = false }

 		
 		
 		# Gére le style
     	styles = Sketchup.active_model.styles
     	status = styles.add_style "Facade", true
     	styles.selected_style = styles["Facade"]
	    	
	    	     	
		# Crée la vues
		
		view = model.active_view
  		camera = view.camera
		camera.perspective = false
		
		section.activate
		up = camera.up
		eye = camera.eye
  		splane = section.get_plane

		eye_on_splane = eye.project_to_plane splane
		target = camera.target
		target.set!(eye_on_splane) 
  		if VICO.plan_horizontal?(splane)
  			up.set!(1, 0, 0)
		else
  			up.set!(0, 0, 1)
		end
  		camera.set(eye, target, up)
		view.camera = camera
		view.zoom_extents
		model.rendering_options['DisplaySectionPlanes'] = false
		Sketchup.active_model.shadow_info['DisplayShadows']=true
		Sketchup.active_model.shadow_info["DisplayOnGroundPlane"] = false
		Sketchup.active_model.rendering_options["DisplayFog"] = true
		page.update
		pages.selected_page = page
		
		end
		
		model.commit_operation   
		model.rendering_options['DisplaySectionPlanes'] = false
		
	end # Facade
	
end  ## module VICO
#=======================================================================# menu
if not file_loaded?(__FILE__)
	add_separator_to_menu("Plugins")
    UI.menu("Plugins").add_item("Mise à jour des sections"){VICO.mise_a_jour}	
	UI.add_context_menu_handler do |menu|
		if menu == nil then 
			UI.messagebox("Error setting context menu handler")
			return
		end
			
		if VICO.selection_section
			menu.add_item("Facade") {VICO.Facade}
			menu.add_item("Coupe") {VICO.Coupe}
			menu.add_item("Plan") {VICO.Plan}
		end
end

end
#-----------------------------------------------------------------------
file_loaded(__FILE__)